/**	\file SimpleType.cpp
 **	\date  2008-05-26
 **	\author grymse@alhem.net
**/
/*
Copyright (C) 2008  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "SimpleType.h"
#include <Sockets/XmlNode.h>
#include "WsdlException.h"
#include <Sockets/Debug.h>
#include "Split.h"
#include "Wsdl.h"


SimpleType::SimpleType(const Wsdl& wsdl, const Xml::XmlNode& node, const std::string& name, const std::string& ns_prefix)
: TypeBase(wsdl, ns_prefix, Split(name).Type)
, m_node(node)
, m_name(!name.empty() ? name : node.PropertyExists("name") ? node.GetProperty("name") : "")
{
	Xml::XmlNode n(node, "restriction");
	if (n)
	{
		if (n.PropertyExists("base"))
		{
			m_type = n.GetProperty("base");
			SetType(m_type);
		}
		Xml::XmlNode n(n, "enumeration");
		while (n)
		{
			std::string value = n.GetProperty("value");
			std::string doc = TypeBase::Documentation(n);
			m_enum.push_back(std::pair<std::string, std::string>(value, doc));
			++n;
		}
	}
}


SimpleType::~SimpleType()
{
}


const std::string SimpleType::Documentation() const
{
	return TypeBase::Documentation(m_node);
}


void SimpleType::CreateInterface(FILE *fil, int lvl) const
{
	for (int i = 0; i < lvl; i++)
		fprintf(fil, "\t");
	fprintf(fil, "typedef %s %s;\n\n", CType().c_str(), Split(m_name).CName.c_str());
}


const std::string SimpleType::CType() const
{
	return TypeBase::CType(/*m_type*/);
}


